import produce from "immer";
import { catchError, concatMap, debounceTime, delay, exhaustMap, flatMap, from, iif, map, mapTo, merge, mergeMap, of, retryWhen, Subject, switchMap, take, takeUntil, tap, throwError } from "rxjs";
import { ModuleUtils } from "../../../utils/lang";
import { ProcessingType } from "../../common/model";
export const initialStateAsyncCall = {
  params: {},
  status: {},
  errors: []
};
export const getTypeOperation = processingType => {
  switch (processingType) {
    case ProcessingType.CONCAT:
      return concatMap;

    case ProcessingType.EXHAUST:
      return exhaustMap;

    case ProcessingType.FLAT:
      return flatMap;

    case ProcessingType.MERGE:
      return mergeMap;

    case ProcessingType.SWITCH:
      return switchMap;

    default:
      return switchMap;
  }
};
export class AsyncCall {
  constructor(executeFn$, config, setState, destroy$) {
    this.executeFn$ = executeFn$;
    this.config = config;
    this.setState = setState;
    this.destroy$ = destroy$;
    this.execute$ = new Subject();
    this.newState = initialStateAsyncCall;
    this.setData$Subject = new Subject();
    this.destroy$ = destroy$ ? destroy$ : new Subject();
    this.init = this.init.bind(this);
    this.getState = this.getState.bind(this);
    this.reload = this.reload.bind(this);
    this.initSetData = this.initSetData.bind(this);
    this.getStatus = this.getStatus.bind(this);
  }

  getStatus() {
    return this.newState.status.status;
  }

  initSetData() {
    this.setData$Subject.pipe(takeUntil(this.destroy$), tap(setData => {
      this.setState(prevState => {
        const newState = produce(draft => {
          const state = this.getState();
          debugger;
          draft.status.data = setData(state.status.data);
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  init() {
    this.initSetData();
    const processingStatus$ = new Subject();
    const config = this.config ? this.config : {};
    const params$ = config.source$ ? merge(this.config.source$, this.execute$) : this.execute$;
    const asyncOperation = getTypeOperation(config.processingType);
    merge(processingStatus$.pipe(mapTo({
      status: 'PROCESSING'
    })), params$.pipe(takeUntil(this.destroy$), debounceTime(10), tap(params => {
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.params = params;
        })(prevState);
        this.newState = newState;
        return newState;
      });
      processingStatus$.next();
    }), asyncOperation(params => {
      const data$ = this.executeFn$ ? this.executeFn$(params) : of(params);
      const loadData$ = ModuleUtils.isObservable(data$) ? data$ : ModuleUtils.isPromise(data$) ? from(data$) : of(data$);
      const retryConfig = config.retry ? config.retry : {
        interval: 1000,
        maxRetryAttempts: 0
      };
      const maxRetryAttempts = retryConfig.maxRetryAttempts ? retryConfig.maxRetryAttempts : 0;
      const retryWhenStatus = retryConfig.retryWhenStatus ? retryConfig.retryWhenStatus : [];
      const notRetryWhenStatus = retryConfig.notRetryWhenStatus ? retryConfig.notRetryWhenStatus : [];
      return loadData$.pipe(retryWhen(errors => errors.pipe(tap(e => {
        this.setState(prevState => {
          const newState = produce(draft => {
            draft.errors.push(e);
          })(prevState);
          this.newState = newState;
          return newState;
        });
      }), concatMap((e, i) => iif(() => {
        if (i < maxRetryAttempts) {
          const responseStatus = (e || {}).status || 0;
          const retry = !!retryWhenStatus.find(status => status === responseStatus);

          if (retry) {
            return false;
          }

          const notRetry = !!notRetryWhenStatus.find(status => status === responseStatus);

          if (notRetry) {
            return true;
          }

          if (retryConfig.retryWhen && retryConfig.retryWhen(e)) {
            return false;
          }

          if (retryConfig.noRetryWhen && retryConfig.noRetryWhen(e)) {
            return true;
          }

          return false;
        }

        return true;
      }, throwError(e), of(e).pipe(delay((retryConfig.typeInterval === 'LINEAR' && 1 || retryConfig.typeInterval === 'EXPONENTIAL' && i) * retryConfig.interval)))))), take(1), map(data => {
        return {
          status: 'SUCCESS',
          data
        };
      }), catchError(error => {
        return of({
          status: 'ERROR',
          error
        });
      }));
    }))).pipe(takeUntil(this.destroy$), tap(processingStatus => {
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.status.data = processingStatus.data;
          draft.status.error = processingStatus.error;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    }), tap(processingStatus => {
      const params = this.newState.params;
      processingStatus.status === 'PROCESSING' && config.onProcessing && config.onProcessing(params);
      processingStatus.status === 'ERROR' && config.onError && config.onError(processingStatus.error, params);
      processingStatus.status === 'SUCCESS' && config.onSuccess && config.onSuccess(processingStatus.data, params);
      this.setState(prevState => {
        const newState = produce(draft => {
          draft.status.status = processingStatus.status;
        })(prevState);
        this.newState = newState;
        return newState;
      });
    })).subscribe();
  }

  getState() {
    return this.newState;
  }

  reload() {
    this.execute$.next(this.newState.params);
  }

  get execute() {
    return this.execute$.next.bind(this.execute$);
  }

  get setData() {
    return this.setData$Subject.next.bind(this.setData$Subject);
  }

}