/**
 * @license
 * Copyright Google LLC All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
import { Observable } from 'rxjs';
import { AsyncValidatorFn, ValidationErrors, ValidatorFn } from './core/validators';
export declare const VALID = "VALID";
export declare const INVALID = "INVALID";
export declare const PENDING = "PENDING";
export declare const DISABLED = "DISABLED";
export declare type FormControlStatus = 'VALID' | 'INVALID' | 'PENDING' | 'DISABLED';
export declare type FormHooks = 'change' | 'blur' | 'submit';
export interface AbstractControlOptions {
    validators?: ValidatorFn | ValidatorFn[] | null;
    asyncValidators?: AsyncValidatorFn | AsyncValidatorFn[] | null;
    updateOn?: 'change' | 'blur' | 'submit';
}
export declare abstract class AbstractControl {
    _pendingDirty: boolean;
    _hasOwnPendingAsyncValidator: boolean;
    _pendingTouched: boolean;
    _onCollectionChange: () => void;
    _updateOn: FormHooks;
    private _parent;
    private _asyncValidationSubscription;
    private _composedValidatorFn;
    private _composedAsyncValidatorFn;
    private _rawValidators;
    private _rawAsyncValidators;
    readonly value: any;
    constructor(validators: ValidatorFn | ValidatorFn[] | null, asyncValidators: AsyncValidatorFn | AsyncValidatorFn[] | null);
    get validator(): ValidatorFn | null;
    set validator(validatorFn: ValidatorFn | null);
    get asyncValidator(): AsyncValidatorFn | null;
    set asyncValidator(asyncValidatorFn: AsyncValidatorFn | null);
    get parent(): FormGroup | FormArray | null;
    readonly status: FormControlStatus;
    get valid(): boolean;
    get invalid(): boolean;
    get pending(): boolean;
    get disabled(): boolean;
    get enabled(): boolean;
    readonly errors: ValidationErrors | null;
    readonly pristine: boolean;
    get dirty(): boolean;
    readonly touched: boolean;
    get untouched(): boolean;
    readonly valueChanges: Observable<any>;
    readonly statusChanges: Observable<FormControlStatus>;
    get updateOn(): FormHooks;
    setValidators(validators: ValidatorFn | ValidatorFn[] | null): void;
    setAsyncValidators(validators: AsyncValidatorFn | AsyncValidatorFn[] | null): void;
    addValidators(validators: ValidatorFn | ValidatorFn[]): void;
    addAsyncValidators(validators: AsyncValidatorFn | AsyncValidatorFn[]): void;
    removeValidators(validators: ValidatorFn | ValidatorFn[]): void;
    removeAsyncValidators(validators: AsyncValidatorFn | AsyncValidatorFn[]): void;
    hasValidator(validator: ValidatorFn): boolean;
    hasAsyncValidator(validator: AsyncValidatorFn): boolean;
    clearValidators(): void;
    clearAsyncValidators(): void;
    markAsTouched(opts?: {
        onlySelf?: boolean;
    }): void;
    markAllAsTouched(): void;
    markAsUntouched(opts?: {
        onlySelf?: boolean;
    }): void;
    markAsDirty(opts?: {
        onlySelf?: boolean;
    }): void;
    markAsPristine(opts?: {
        onlySelf?: boolean;
    }): void;
    markAsPending(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    disable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    enable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    private _updateAncestors;
    setParent(parent: FormGroup | FormArray): void;
    abstract setValue(value: any, options?: Object): void;
    abstract patchValue(value: any, options?: Object): void;
    abstract reset(value?: any, options?: Object): void;
    updateValueAndValidity(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    _updateTreeValidity(opts?: {
        emitEvent?: boolean;
    }): void;
    private _setInitialStatus;
    private _runValidator;
    private _runAsyncValidator;
    private _cancelExistingSubscription;
    setErrors(errors: ValidationErrors | null, opts?: {
        emitEvent?: boolean;
    }): void;
    get(path: Array<string | number> | string): AbstractControl | null;
    getError(errorCode: string, path?: Array<string | number> | string): any;
    hasError(errorCode: string, path?: Array<string | number> | string): boolean;
    get root(): AbstractControl;
    /** @internal */
    _updateControlsErrors(emitEvent: boolean): void;
    /** @internal */
    _initObservables(): void;
    private _calculateStatus;
    /** @internal */
    abstract _updateValue(): void;
    /** @internal */
    abstract _forEachChild(cb: Function): void;
    /** @internal */
    abstract _anyControls(condition: Function): boolean;
    /** @internal */
    abstract _allControlsDisabled(): boolean;
    /** @internal */
    abstract _syncPendingControls(): boolean;
    /** @internal */
    _anyControlsHaveStatus(status: FormControlStatus): boolean;
    /** @internal */
    _anyControlsDirty(): boolean;
    /** @internal */
    _anyControlsTouched(): boolean;
    /** @internal */
    _updatePristine(opts?: {
        onlySelf?: boolean;
    }): void;
    /** @internal */
    _updateTouched(opts?: {
        onlySelf?: boolean;
    }): void;
    /** @internal */
    _onDisabledChange: Function[];
    /** @internal */
    _isBoxedValue(formState: any): boolean;
    /** @internal */
    _registerOnCollectionChange(fn: () => void): void;
    /** @internal */
    _setUpdateStrategy(opts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null): void;
    private _parentMarkedDirty;
    nestedPath: string;
    path: string | number | null;
    dispatchStateFn: (value: any) => void;
    getRoot(): AbstractControl;
    setState(value: any): void;
    getPath(): (string | number | null)[];
    getPathString(): string;
}
export declare class FormControl extends AbstractControl {
    _onChange: Function[];
    _pendingValue: any;
    _pendingChange: any;
    constructor(formState?: any, validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null);
    setValue(value: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    patchValue(value: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    reset(formState?: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    _updateValue(): void;
    _anyControls(condition: Function): boolean;
    _allControlsDisabled(): boolean;
    registerOnChange(fn: Function): void;
    _unregisterOnChange(fn: Function): void;
    registerOnDisabledChange(fn: (isDisabled: boolean) => void): void;
    _unregisterOnDisabledChange(fn: (isDisabled: boolean) => void): void;
    _forEachChild(cb: Function): void;
    _syncPendingControls(): boolean;
    private _applyFormState;
    onChange: (event: any) => void;
    onBlur: () => void;
}
export declare class FormGroup extends AbstractControl {
    controls: {
        [key: string]: AbstractControl;
    };
    constructor(controls: {
        [key: string]: AbstractControl;
    }, validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null);
    registerControl(name: string, control: AbstractControl): AbstractControl;
    addControl(name: string, control: AbstractControl, options?: {
        emitEvent?: boolean;
    }): void;
    removeControl(name: string, options?: {
        emitEvent?: boolean;
    }): void;
    setControl(name: string, control: AbstractControl, options?: {
        emitEvent?: boolean;
    }): void;
    contains(controlName: string): boolean;
    setValue(value: {
        [key: string]: any;
    }, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    patchValue(value: {
        [key: string]: any;
    }, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    reset(value?: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    getRawValue(): any;
    _syncPendingControls(): boolean;
    _throwIfControlMissing(name: string): void;
    _forEachChild(cb: (v: any, k: string) => void): void;
    _setUpControls(): void;
    _updateValue(): void;
    _anyControls(condition: Function): boolean;
    _reduceValue(): any;
    /** @internal */
    _reduceChildren(initValue: any, fn: Function): any;
    _allControlsDisabled(): boolean;
    /** @internal */
    _checkAllValuesPresent(value: any): void;
}
export declare class FormArray extends AbstractControl {
    controls: AbstractControl[];
    constructor(controls: AbstractControl[], validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null);
    at(index: number): AbstractControl;
    push(control: AbstractControl, options?: {
        emitEvent?: boolean;
    }): void;
    insert(index: number, control: AbstractControl, options?: {
        emitEvent?: boolean;
    }): void;
    removeAt(index: number, options?: {
        emitEvent?: boolean;
    }): void;
    setControl(index: number, control: AbstractControl, options?: {
        emitEvent?: boolean;
    }): void;
    get length(): number;
    setValue(value: any[], options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    patchValue(value: any[], options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    reset(value?: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    getRawValue(): any[];
    clear(options?: {
        emitEvent?: boolean;
    }): void;
    _syncPendingControls(): boolean;
    _throwIfControlMissing(index: number): void;
    _forEachChild(cb: Function): void;
    _updateValue(): void;
    _anyControls(condition: Function): boolean;
    _setUpControls(): void;
    _checkAllValuesPresent(value: any): void;
    _allControlsDisabled(): boolean;
    private _registerControl;
}
export declare const isReactNative: () => boolean;
export declare const isEvent: (candidate: any) => boolean;
